#!/usr/bin/env python

import logging
from pathlib import Path

import torch
from lerobot.common.datasets.lerobot_dataset import LeRobotDataset

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

def main():
    # Initialize the dataset
    dataset = LeRobotDataset(
        repo_id="aractingi/pushcube_gamepad",
        download_videos=True,  # Set to False if you don't need video data
    )
    
    # Print dataset information
    logger.info(f"Dataset loaded successfully!")
    logger.info(f"Number of episodes: {dataset.num_episodes}")
    logger.info(f"Number of frames: {dataset.num_frames}")
    logger.info(f"FPS: {dataset.fps}")
    logger.info(f"Features: {list(dataset.features.keys())}")
    
    # Get a sample frame
    sample = dataset[0]
    logger.info(f"\nSample frame keys: {list(sample.keys())}")
    
    # Print shapes of some key features
    for key in ["observation.images.laptop", "observation.images.phone"]:
        if key in sample:
            logger.info(f"Shape of {key}: {sample[key].shape}")
    
    # Print task information
    logger.info(f"\nTotal tasks: {dataset.meta.total_tasks}")
    logger.info("Tasks:")
    for task_idx, task in dataset.meta.tasks.items():
        logger.info(f"  {task_idx}: {task}")
    global_min = float("inf")
    global_max = float("-inf")
    for sample in dataset:
        for k, v in sample.items():
            if isinstance(v, torch.Tensor):
                v = v.to("cuda")
            if k == "observation.state":
                global_min = min(global_min, torch.min(v))
                global_max = max(global_max, torch.max(v))
    print(global_min, global_max)

    breakpoint()   

if __name__ == "__main__":
    main() 